//test measurement sequentially, reading an increasing number of channels
#include <Wire.h>
#include <dummy.h> 
#define SLAVE 0x50
#define NUM_SENSORS 4
int OFFSET = 15; 
int capdac = round(OFFSET / 3.125); 
double data_list[4] = { };
int check1[2];
int check_1[4];
int Coffset= capdac * 3.125;
String capdac_hex = String(capdac, HEX);

void setup() {
  delay(1000); 
  Wire.begin(21, 22);
  Serial.begin(115200);

  //reset the first FDC 
  Wire.beginTransmission(SLAVE);
  Wire.write(0x0C);
  Wire.write(0x80); 
  Wire.endTransmission();

  Serial.println(""); 
  Serial.println("Capdac as int:  ");
  Serial.println(capdac);
  Serial.println(""); 
  Serial.println("Capdac in hexadecimal already:  ");
  Serial.println(capdac_hex);
  Serial.println(""); 
  Serial.println("Capdac with hexadecimal conversion:  ");
  Serial.println(capdac, HEX);
  Serial.println(""); 

  //Initiate CIN 1-4 for single ended measurement with offset of 15.625 pF
  //Note that parasitic capacitances between CIN_N and GND shift the measured value by roughly 1.5 pF in the current setup
  for (int i = 0; i < 4; i++) {
    int register_value = 0x1000 + (capdac << 5); 
    // Print the decimal value
    Serial.println("");
    Serial.print("Decimal value: ");
    Serial.println(register_value);
    // Print the binary value
    Serial.println("");
    Serial.print("Binary value: ");
    for (int i = 15; i >= 0; i--) {
      if (register_value & (1 << i)) {
        Serial.print("1");
      } else {
        Serial.print("0");
      }
    }
    Serial.println();
    Serial.println(""); 
    uint8_t reg_highbyte = (register_value >> 8) & 0xFF;  
    Serial.println(""); 
    Serial.print("Register Value high byte: ");
    for (int i = 7; i >= 0; i--) {
      if (reg_highbyte & (1 << i)) {
        Serial.print("1");
      } else {
        Serial.print("0");
      }
    }
    Serial.println(""); 
    uint8_t reg_lowbyte = register_value & 0xFF;
    Serial.println(""); 
    Serial.println("Register value low byte: ");
    Serial.println(reg_lowbyte, BIN);
    Serial.println(""); 

    //set-up configuration measurement register
    Wire.beginTransmission(SLAVE); //the configuration register is composed of 16 bit
    Wire.write(0x08 + i); //address the measurement config register for the 4 channels: 0x08,0x09,0x0A,0x0B
    Wire.write(reg_highbyte + 32 * i);
    Wire.write(reg_lowbyte);
    Wire.endTransmission();
  }

  //Check register readout to see if they are set-up correctly
  for (int i = 0; i < NUM_SENSORS; i++) {
    Wire.beginTransmission(SLAVE); 
    Wire.write(0x08 + i); 
    Wire.endTransmission();
    Wire.requestFrom(SLAVE, 2);
    for (int j = 0; j < 2; j++) {
      check1[j] = Wire.read();
    }
    check_1[i] = (check1[0] << 8) + check1[1];
  }
  Serial.println(""); 
  Serial.println("Readback initialization registers: ");
  for (int i = 0; i < NUM_SENSORS; i++) {
    int bitsCount = sizeof( check_1[i] ) * 8;
    char str[ bitsCount + 1 ];
    uint8_t j = 0;
    while ( bitsCount-- )
      str[ j++ ] = bitRead( check_1[i], bitsCount ) + '0';
    str[ j ] = '\0';
    Serial.print("Check1_" + String(i+1) + ":  ");
    Serial.println(str); // + (str, BIN) + " " );
  }
  Serial.println("");

  //Initiate measurements with data rate of 100S/s and continuous measurements
  Wire.beginTransmission(SLAVE);
  Wire.write(0x0C); 
  Wire.write(0x05); 
  Wire.write(0xF0); 
  Wire.endTransmission();
}

void loop() {
  Serial.println(""); 
  if (isReady(0)) {
    data_list[0] = readC(0);
  }
  if (isReady(1)) {
    data_list[1] = readC(1);
  }
  if (isReady(2)) {
    data_list[2] = readC(2);
  }
  if (isReady(3)) {
    data_list[3] = readC(3);
  }
  for (int i = 0; i < 4; i++) {
    Serial.print("CH" + String(i+1) + ":" + String(data_list[i], 5) + " " );
  }
  Serial.println("");
}

//check whether conversion result is available
bool isReady(int ch_n) { 
  int crash = 0;
  crash++;                                      
  if (crash > 20) Serial.println("error");
  int fdc[4];
  int i = 0;
  Wire.beginTransmission(SLAVE);
  Wire.write(0x0C); //address the FDC configuration register 
  Wire.endTransmission();
  Wire.requestFrom(SLAVE, 4); 
  while (Wire.available()) {
    fdc[i] = Wire.read();
    if (crash > 20) Serial.print(fdc[i], BIN); 
    i++;
  }
  return bitRead(fdc[1], 3 - ch_n);
}

//Read Measurement results 
double readC(int ch_n) {
  int c_data[4];
  int32_t c;
  Wire.beginTransmission(SLAVE);
  Wire.write(0x00 + ch_n * 2); 
  Wire.endTransmission();
  Wire.requestFrom(SLAVE, 4); 
  for (int i = 0; i < 4; i++) {
    c_data[i] = Wire.read();  
  }
    for (int i = 0; i < NUM_SENSORS; i++) {
    //Serial.print("CH" + String(i+5) + ": " + (data_list2[i], BIN) + " " );
    int bitsCount = sizeof( c_data[i] ) * 8;
    char str[ bitsCount + 1 ];
    uint8_t j = 0;
    while ( bitsCount-- )
      str[ j++ ] = bitRead( c_data[i], bitsCount ) + '0';
    str[ j ] = '\0';
    Serial.print("Byte" + String(i+1) + ":  ");
    Serial.println(str);
  }
  Serial.println("");
  
  //compose the 24-bits value omitting the 8 least significan bits, that should all be zeros 
  c = (c_data[0] << 16) + (c_data[1] << 8) + c_data[2]; // MSB always first, LSB after
  
  //if the value is negative perform the 2's complement
  if ((c & 0x800000) != 0) { // Check if value is negative --> if MSb==0 it's positive, if MSb !=0 it's negative
    c = ~c + 1; // Invert all bits and add 1
  }
  double cap = ((float)(c)/ 524288.0)+ Coffset;
  return cap;
}

